﻿using System;
using System.Drawing;
using System.Windows.Forms;
using MapGenerator;

namespace MapGeneratorGUI
{
	public partial class MapGenerateForm : Form
	{
		private readonly Bitmap[] images = new Bitmap[6];
		private Map map;

		public MapGenerateForm()
		{
			InitializeComponent();
		}

		protected override void OnLoad(EventArgs e)
		{
			this.ActiveControl = this.generateButton;
			base.OnLoad(e);
		}

		private void DrawMap()
		{
			int index = this.imageMagnificationTrackBar.Value - 1;
			if (this.images[index] == null)
			{
				this.images[index] = this.map.GetImage(this.imageMagnificationTrackBar.Value);
			}
			if (this.images[index] != this.panel1.BackgroundImage)
			{
				this.panel1.BackgroundImage = this.images[index];
				this.panel1.ClientSize = this.images[index].Size;
			}
		}

		private void generateButton_Click(object sender, EventArgs e)
		{
			try
			{
				this.map = new Map((ushort)this.widthNumericUpDown.Value, (ushort)this.heightNumericUpDown.Value, (byte)this.leftSpaceNumericUpDown.Value, (byte)this.topSpaceNumericUpDown.Value, (byte)this.rightSpaceNumericUpDown.Value, (byte)this.bottomSpaceNumericUpDown.Value, (byte)this.minDivWidthNumericUpDown.Value, (byte)this.minDivHeightNumericUpDown.Value, (byte)this.maxDivWidthNumericUpDown.Value, (byte)this.maxDivHeightNumericUpDown.Value, (byte)this.minRoomWidthNumericUpDown.Value, (byte)this.minRoomHeightNumericUpDown.Value, (byte)this.maxRoomWidthNumericUpDown.Value, (byte)this.maxRoomHeightNumericUpDown.Value, (ushort)this.connectionAdditionRateNumericUpDown.Value, (uint)this.seedXNumericUpDown.Value, (uint)this.seedYNumericUpDown.Value, (uint)this.seedZNumericUpDown.Value, (uint)this.seedWNumericUpDown.Value);
			}
			catch (ArgumentOutOfRangeException ex)
			{
				MessageBox.Show(ex.Message, "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			this.generateButton.Enabled = this.widthNumericUpDown.Enabled = this.heightNumericUpDown.Enabled = this.leftSpaceNumericUpDown.Enabled = this.topSpaceNumericUpDown.Enabled = this.rightSpaceNumericUpDown.Enabled = this.bottomSpaceNumericUpDown.Enabled = this.minDivWidthNumericUpDown.Enabled = this.minDivHeightNumericUpDown.Enabled = this.maxDivWidthNumericUpDown.Enabled = this.maxDivHeightNumericUpDown.Enabled = this.minRoomWidthNumericUpDown.Enabled = this.minRoomHeightNumericUpDown.Enabled = this.maxRoomWidthNumericUpDown.Enabled = this.maxRoomHeightNumericUpDown.Enabled = this.connectionAdditionRateNumericUpDown.Enabled = this.seedXNumericUpDown.Enabled = this.seedYNumericUpDown.Enabled = this.seedZNumericUpDown.Enabled = this.seedWNumericUpDown.Enabled = false;
			this.destructButton.Enabled = this.saveImageButton.Enabled = true;
			if (this.seedXNumericUpDown.Value == 0 && this.seedYNumericUpDown.Value == 0 && this.seedZNumericUpDown.Value == 0 && this.seedWNumericUpDown.Value == 0)
			{
				this.seedXNumericUpDown.Value = this.map.SeedX;
				this.seedYNumericUpDown.Value = this.map.SeedY;
				this.seedZNumericUpDown.Value = this.map.SeedZ;
				this.seedWNumericUpDown.Value = this.map.SeedW;
			}
			this.DrawMap();
		}

		private void destructButton_Click(object sender, EventArgs e)
		{
			this.map = null;
			for (int i = 0; i < 6; ++i)
			{
				this.images[i] = null;
			}
			this.panel1.ClientSize = new Size(0, 0);
			this.panel1.BackgroundImage = null;
			this.seedXNumericUpDown.Value = this.seedYNumericUpDown.Value = this.seedZNumericUpDown.Value = this.seedWNumericUpDown.Value = 0;
			this.destructButton.Enabled = this.saveImageButton.Enabled = false;
			this.generateButton.Enabled = this.widthNumericUpDown.Enabled = this.heightNumericUpDown.Enabled = this.leftSpaceNumericUpDown.Enabled = this.topSpaceNumericUpDown.Enabled = this.rightSpaceNumericUpDown.Enabled = this.bottomSpaceNumericUpDown.Enabled = this.minDivWidthNumericUpDown.Enabled = this.minDivHeightNumericUpDown.Enabled = this.maxDivWidthNumericUpDown.Enabled = this.maxDivHeightNumericUpDown.Enabled = this.minRoomWidthNumericUpDown.Enabled = this.minRoomHeightNumericUpDown.Enabled = this.maxRoomWidthNumericUpDown.Enabled = this.maxRoomHeightNumericUpDown.Enabled = this.connectionAdditionRateNumericUpDown.Enabled = this.seedXNumericUpDown.Enabled = this.seedYNumericUpDown.Enabled = this.seedZNumericUpDown.Enabled = this.seedWNumericUpDown.Enabled = true;
		}

		private void saveImageButton_Click(object sender, EventArgs e)
		{
			if (this.saveImageDialog.ShowDialog() == DialogResult.OK)
			{
				this.images[this.imageMagnificationTrackBar.Value - 1].Save(this.saveImageDialog.FileName, System.Drawing.Imaging.ImageFormat.Png);
			}
		}

		private void imageMagnificationTrackBar_Scroll(object sender, EventArgs e)
		{
			this.imageMagnificationLabel.Text = this.imageMagnificationTrackBar.Value.ToString();
			if (this.map != null)
			{
				this.DrawMap();
			}
		}
	}
}
